MODULE Scratch;

(* Translation of Raymond Chen's Scratch program

   http://blogs.msdn.com/b/oldnewthing/

   2014. Ported to Modula-2 by Frank Schoonjans

*)

FROM WIN32    IMPORT *; (* import all *)
FROM WINUSER  IMPORT MoveWindow,LPCREATESTRUCT,PostQuitMessage,PAINTSTRUCT,BeginPaint,
                     EndPaint,GetClientRect,WS_OVERLAPPEDWINDOW,
                     CW_USEDEFAULT,WNDCLASS,LoadCursor,IDC_ARROW,COLOR_WINDOW,
                     DefWindowProc,RegisterClass,MSG,ShowWindow,CreateWindow,
                     GetMessage,TranslateMessage,DispatchMessage,
                     WM_CREATE,WM_SIZE,WM_DESTROY,WM_PAINT,WM_PRINTCLIENT,
                     LOWORD,HIWORD,MessageBox;
FROM WINERROR IMPORT SUCCEEDED;
FROM WINGDI   IMPORT TextOutW;
FROM COMMCTRL IMPORT InitCommonControls;

FROM WINX     IMPORT NULL_HWND,NULL_HICON,NULL_HINSTANCE,NULL_HMENU;

FROM SYSTEM   IMPORT CAST,ADR,FUNC;

IMPORT OOle2,WINX;

VAR

 ClassName   : ARRAY [0..31] OF CHAR;

 g_hinst     : HINSTANCE;      (* This application's HINSTANCE *)
 g_hwndChild : HWND;           (* Optional child window *)


PROCEDURE Alert(txt : ARRAY OF CHAR);
BEGIN
     FUNC MessageBox(NULL_HWND,txt,"Scratch",0);
END Alert;

PROCEDURE AlertError(errornr : INTEGER);
VAR str : ARRAY [0..255] OF CHAR;
BEGIN
    FUNC FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM BOR FORMAT_MESSAGE_IGNORE_INSERTS,
                        NIL,errornr,0,str,HIGH(str)+1,NIL);
    Alert(str);
END AlertError;


(*  OnSize
        If we have an inner child, resize it to fit. *)
PROCEDURE OnSize(hwnd : HWND; state : UINT; cx,cy : INTEGER);
BEGIN
    IF (g_hwndChild#NULL_HWND) THEN
        FUNC MoveWindow(g_hwndChild, 0, 0, cx, cy, TRUE);
    END;
END OnSize;

(*  OnCreate
        Applications will typically override this and maybe even
        create a child window. *)
PROCEDURE OnCreate(hwnd : HWND; lpcs : LPCREATESTRUCT) : BOOL;
BEGIN
    RETURN TRUE;
END OnCreate;

(*  OnDestroy
        Post a quit message because our application is over when the
        user closes this window.  *)
PROCEDURE OnDestroy(hwnd : HWND);
BEGIN
    PostQuitMessage(0);
END OnDestroy;

(*  PaintContent
        Interesting things will be painted here eventually.  *)
PROCEDURE PaintContent(hwnd : HWND; VAR ps: PAINTSTRUCT);
BEGIN
(*
    TextOut(ps.hdc, 0, 0,
            "ABC"+CHR(0410H)+CHR(0411H)+CHR(0412H)+CHR(0E01H)+CHR(0E02H)+CHR(0E03H), 9 );
    *)
    TextOutW(ps.hdc, 0, 0, UCHR(65)+UCHR(66)+UCHR(67)+UCHR(0410H)+UCHR(0411H)+UCHR(0412H)+UCHR(0E01H)+UCHR(0E02H)+UCHR(0E03H), 9);
END PaintContent;

(*  OnPaint
        Paint the content as part of the paint cycle.  *)
PROCEDURE OnPaint(hwnd : HWND);
VAR ps : PAINTSTRUCT;
BEGIN
    FUNC BeginPaint(hwnd, ps);
    PaintContent(hwnd, ps);
    FUNC EndPaint(hwnd, ps);
END OnPaint;

(*  OnPrintClient
        Paint the content as requested by USER.  *)
PROCEDURE OnPrintClient(hwnd : HWND; hdc : HDC);
VAR ps : PAINTSTRUCT;
BEGIN
    ps.hdc := hdc;
    FUNC GetClientRect(hwnd, ps.rcPaint);
    PaintContent(hwnd, ps);
END OnPrintClient;


(*  Window procedure  *)
<*/PUSH*>
<*/CALLS:WINDOWSCALL*>
<*/NOHIGH*>
<*/ALIGN:8/NOPACK*>
PROCEDURE WndProc(hwnd : HWND; uiMsg : UINT; wParam : WPARAM; lParam : LPARAM) : LRESULT [EXPORT];
BEGIN
    CASE uiMsg OF
    | WM_CREATE      : RETURN VAL(LRESULT,~OnCreate(hwnd,CAST(LPCREATESTRUCT,lParam)));
    | WM_SIZE        : OnSize(hwnd,wParam,LOWORD(lParam),HIWORD(lParam));
    | WM_DESTROY     : OnDestroy(hwnd);
    | WM_PAINT       : OnPaint(hwnd);
    | WM_PRINTCLIENT : OnPrintClient(hwnd, CAST(HDC,wParam));
    ELSE  RETURN DefWindowProc(hwnd, uiMsg, wParam, lParam);
    END;
    RETURN 0;
END WndProc;
<*/POP*>

PROCEDURE InitApp() : BOOL;
VAR wc : WNDCLASS;
BEGIN
    wc.style         := 0;
    wc.lpfnWndProc   := WndProc;
    wc.cbClsExtra    := 0;
    wc.cbWndExtra    := 0;
    wc.hInstance     := g_hinst;
    wc.hIcon         := NULL_HICON;
    wc.hCursor       := LoadCursor(NULL_HINSTANCE, IDC_ARROW^);
    wc.hbrBackground := CAST(HBRUSH,COLOR_WINDOW + 1);
    wc.lpszMenuName  := NIL;
    wc.lpszClassName := ADR(ClassName);

    IF RegisterClass(wc)=0 THEN AlertError(GetLastError()); RETURN FALSE END;

    InitCommonControls();            (* In case we use a common control *)

    RETURN TRUE;
END InitApp;


VAR
    msg  : MSG;
    hwnd : HWND;
BEGIN

    g_hinst     := WINX.Instance;
    g_hwndChild := NULL_HWND;

    %IF UNICODE %THEN
        Alert("UNICODE");
    %ELSE
        Alert("ANSI");
    %END

    ClassName:="Scratch";

    IF NOT InitApp() THEN Alert("Init failed"); HALT END;

    IF SUCCEEDED(OOle2.CoInitialize(NIL)) THEN (* In case we use COM *)

        hwnd := CreateWindow(
            ClassName,                      (* Class Name *)
            ClassName,                      (* Window Title *)
            WS_OVERLAPPEDWINDOW,            (* Style *)
            CW_USEDEFAULT, CW_USEDEFAULT,   (* Position *)
            CW_USEDEFAULT, CW_USEDEFAULT,   (* Size *)
            NULL_HWND,                      (* Parent *)
            NULL_HMENU,                     (* No menu *)
            WINX.Instance,                  (* Instance *)
            NIL);                           (* No special parameters *)

        IF hwnd=NULL_HWND THEN AlertError(GetLastError()); HALT; END;

        FUNC ShowWindow(hwnd, WINX.CmdShow );

        WHILE GetMessage(msg, NIL, 0, 0) DO
            FUNC TranslateMessage(msg);
            FUNC DispatchMessage(msg);
        END;

        OOle2.CoUninitialize();

    END;
END Scratch.
